PROGRAM freiburg;

{$I OsWahl.inc}

USES Stdio, Stack, Liste;

TYPE
   PEintrag = ^TEintrag;
   TEintrag = RECORD
      M1, M2, M3, N1, N2, L1: INTEGER;
      PROG, BEZEICHNER, CODE: STRING[50];
   END;

   PMEintrag = {$IFDEF DOS} PListeElement {$ELSE} specialize TListe<TEintrag>.PListeElement {$ENDIF};
   TMListe   = {$IFDEF DOS} TListe {$ELSE} specialize TListe<TEintrag> {$ENDIF};

   TMenueManager = OBJECT
      Gesamt, Auswahl: TMListe;
      Aktuell, Punkt: TEintrag;
      MenueStapel: {$IFDEF DOS} TStack {$ELSE} specialize TStack<TEintrag> {$ENDIF};
      Angezeigt: BOOLEAN;
      PROCEDURE Init;
      PROCEDURE Freigeben;
      PROCEDURE LoadCSV(CONST FileName: STRING);
      PROCEDURE ZeigeMenue;
      PROCEDURE Inhalt(Zeiger: PMEintrag; VAR Eintrag: TEintrag);
      FUNCTION  BearbeiteAuswahl: BOOLEAN;
      FUNCTION  FindeMenuepunkt(M1, M2, M3: INTEGER): BOOLEAN;
      FUNCTION  HatNachfolge(N1, N2: INTEGER): BOOLEAN;
      FUNCTION  AktuelleMenuepunkte: BOOLEAN;
      PROCEDURE MenueAusfuehren;
   END;

PROCEDURE Init_Eintrag(VAR eintrag: TEintrag);
BEGIN
   WITH Eintrag DO BEGIN
      M1 := 0; M2 := 0; M3 := 0; N1 := 0; N2 := 0; L1 := 0;
      PROG := ''; CODE := ''; BEZEICHNER := '';
   END;
END;

PROCEDURE TMenueManager.Init;
BEGIN
   Gesamt.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};;
   Auswahl.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};;
   Init_Eintrag(Aktuell); Aktuell.BEZEICHNER := 'Hauptmenue';
   Angezeigt := False;
   MenueStapel.Init {$IFDEF DOS} (SizeOf(TEintrag)) {$ENDIF};
END;

PROCEDURE TMenueManager.Freigeben;
BEGIN
   Gesamt.Clear; Auswahl.Clear;
   MenueStapel.Clear;
END;

PROCEDURE TMenueManager.Inhalt(Zeiger: PMEintrag; VAR Eintrag: TEintrag);
{$IFDEF DOS} VAR Temp: PEintrag; {$ENDIF}
BEGIN
   {$IFDEF DOS} Temp := Zeiger^.Data; Eintrag := Temp^; {$ELSE} Eintrag := Zeiger^.Data; {$ENDIF}
END;

FUNCTION TMenueManager.FindeMenuepunkt(M1, M2, M3: INTEGER): BOOLEAN;
VAR Zeiger: PMEintrag;
    Eintrag: TEintrag;
    ok: BOOLEAN;
BEGIN
   ok := False;
   Zeiger := Gesamt.Head;
   WHILE (Zeiger <> NIL) AND NOT ok DO BEGIN
      Inhalt(Zeiger, Eintrag);     
      IF (Eintrag.M1 = M1) AND (Eintrag.M2 = M2) AND (Eintrag.M3 = M3) THEN BEGIN
         Punkt := Eintrag; ok := True;
      END;
      Zeiger := Zeiger^.Next;
   END;
   FindeMenuepunkt := ok;
END;

FUNCTION TMenueManager.HatNachfolge(N1, N2: INTEGER): BOOLEAN;
VAR Zeiger: PMEintrag;
    Eintrag: TEintrag;
    wert: BOOLEAN;
BEGIN
   wert := False;
   Zeiger := Gesamt.Head;
   WHILE (Zeiger <> NIL) AND NOT wert DO BEGIN
      Inhalt(Zeiger, Eintrag);     
      IF (Eintrag.M1 = N1) AND (Eintrag.M2 = N2)
         THEN wert := True;
      Zeiger := Zeiger^.Next;
   END;
   HatNachfolge := wert;
END;

FUNCTION TMenueManager.AktuelleMenuepunkte:BOOLEAN;
VAR Zeiger: PMEintrag;
    Eintrag: TEintrag;
BEGIN
   Auswahl.Clear;
   Zeiger := Gesamt.Head;
   WHILE Zeiger <> NIL DO BEGIN
      Inhalt(Zeiger, Eintrag);     
      IF (Eintrag.M1 = Aktuell.M1) AND (Eintrag.M2 = Aktuell.M2)
         THEN Auswahl.Add({$IFDEF DOS}@{$ENDIF}Eintrag);
      Zeiger := Zeiger^.Next;
   END;
   AktuelleMenuepunkte := Auswahl.Anzahl <> 0;
END;

PROCEDURE TMenueManager.ZeigeMenue;
VAR Zeiger: PMEintrag;
    EIntrag: TEintrag;
    i: INTEGER;
BEGIN
   IF Angezeigt THEN Exit;
   IF NOT AktuelleMenuepunkte THEN
      Writeln('Fehler: Menuezeilen nicht vorhanden.')
   ELSE BEGIN
      Writeln(Aktuell.BEZEICHNER, ':');
      Zeiger := Auswahl.Head;
      WHILE Zeiger <> NIL DO BEGIN
         Inhalt(Zeiger, Eintrag);     
         Writeln(Eintrag.M3, ': ', Eintrag.BEZEICHNER);
         FOR i := 1 TO Eintrag.L1 DO Writeln;
         Zeiger := Zeiger^.Next;
      END;
      Writeln('0: Zurueck');
      Angezeigt := True;
   END;
END;

FUNCTION TMenueManager.BearbeiteAuswahl: BOOLEAN;
VAR wahl: CHAR;
    ziffer,w: INTEGER;
BEGIN
   BearbeiteAuswahl := False;
   Write('Auswahl: '); ReadLn(wahl);
   IF wahl in ['0'..'9'] THEN VAL(wahl, ziffer, w)
   ELSE BEGIN
      Writeln('Ungueltige Auswahl. Bitte 0..9 eingeben');
      Exit;
   END;
   IF ziffer = 0 THEN BEGIN
      Angezeigt := False;
      IF MenueStapel.IsEmpty THEN BEGIN
         Writeln('Programmende.');
         BearbeiteAuswahl := True; Exit;
      END;
      MenueStapel.Pop({$IFDEF DOS}@{$ENDIF}Aktuell);
      Exit;
   END;
   IF NOT FindeMenuepunkt(Aktuell.M1, Aktuell.M2, ziffer) THEN BEGIN
      Writeln('Ungueltige Auswahl. Menuepunkt nicht vorhanden. ');
      Exit;
   END;
   Aktuell.BEZEICHNER := Punkt.BEZEICHNER;
   CASE UpCase(Punkt.PROG[1]) OF
     'P': Writeln('Starte Programm: ', Punkt.CODE);
     'M': IF HatNachfolge(Punkt.N1, Punkt.N2) THEN BEGIN
             MenueStapel.Push({$IFDEF DOS}@{$ENDIF}Aktuell);
             Aktuell.M1 := Punkt.N1; Aktuell.M2 := Punkt.N2;
             Angezeigt := False;
          END
          ELSE Writeln('Ungueltige Auswahl. Aufgerufenes Menue nicht gefunden.');
      ELSE Writeln('Fehler: Falscher Programmcode.');
   END;
END;

PROCEDURE TMenueManager.MenueAusfuehren;
BEGIN
   REPEAT
      ZeigeMenue;
   UNTIL BearbeiteAuswahl;
END;

PROCEDURE TMenueManager.LoadCSV(CONST FileName: STRING);
VAR
   CSVFile: TEXT;
   Eintrag: TEintrag;
   Zeile: STRING;
   FUNCTION Strip: STRING;
   VAR k: INTEGER;
   BEGIN
      k := Pos(';', Zeile); IF k = 0 THEN k := LENGTH(Zeile) + 1;
      Strip := Copy(Zeile, 1, k - 1);
      Zeile := Copy(Zeile, k + 1, 255);
   END;
BEGIN
   Assign(CSVFile, FileName); Reset(CSVFile);
   ReadLn(CSVFile, Zeile);
   WHILE NOT EOF(CSVFile) DO BEGIN
      ReadLn(CSVFile, Zeile);
      WITH Eintrag DO BEGIN
         M1 := Str2Int(Strip); M2 := Str2Int(Strip); M3 := Str2Int(Strip);
         PROG := Strip; BEZEICHNER := Strip; CODE := Strip;
         N1 := Str2Int(Strip); N2 := Str2Int(Strip); L1 := Str2Int(Strip);
      END;
      IF Eintrag.PROG <> 'D' THEN
         Gesamt.Add({$IFDEF DOS}@{$ENDIF}Eintrag);
   END;
   Close(CSVFile);
END;

VAR MenueManager: TMenueManager;

BEGIN
   WITH Menuemanager DO BEGIN
      Init; LoadCSV('daten.csv');
      MenueAusfuehren;
      Freigeben;
   END;
   Writeln('Beenden'); Readln;
END.
